@@ -81,8 +81,6 @@ static void init_env_entries(char ***, int *, const char *, const char *, int);
  * Initialize the shabang
  */
 
-static char log_domain[] = "libsandbox";
-
 __attribute__((destructor))
 void libsb_fini(void)
 {
@@ -115,7 +113,6 @@ void libsb_init(void)
 	mtrace();
 #endif
 
-	rc_log_domain(log_domain);
 	sb_set_open(libsb_open);
 
 	/* Get the path and name to this library */
@@ -309,59 +306,7 @@ char *egetcwd(char *buf, size_t size)
 	return tmpbuf;
 }
 
-static char *getcmdline(void)
-{
-	rc_dynbuf_t *proc_data;
-	struct stat st;
-	char *buf;
-	size_t n;
-	int fd;
-
-	if (-1 == stat(PROC_SELF_CMDLINE, &st)) {
-		/* Don't care if it does not exist */
-		errno = 0;
-		return NULL;
-	}
-
-	proc_data = rc_dynbuf_new();
-
-	fd = sb_open(PROC_SELF_CMDLINE, O_RDONLY, 0);
-	if (fd < 0) {
-		SB_EERROR("ISE ", "Failed to open '%s'!\n", PROC_SELF_CMDLINE);
-		return NULL;
-	}
-
-	/* Read PAGE_SIZE at a time -- whenever EOF or an error is found
-	 * (don't care) give up and return.
-	 * XXX: Some linux kernels especially needed read() to read PAGE_SIZE
-	 *      at a time. */
-	do {
-		n = rc_dynbuf_write_fd(proc_data, fd, getpagesize());
-		if (-1 == n) {
-			SB_EERROR("ISE ", "Failed to read from '%s'!\n", PROC_SELF_CMDLINE);
-			goto error;
-		}
-	} while (0 < n);
-
-	sb_close(fd);
-
-	rc_dynbuf_replace_char(proc_data, '\0', ' ');
-
-	buf = rc_dynbuf_read_line(proc_data);
-	if (NULL == buf)
-		goto error;
-
-	rc_dynbuf_free(proc_data);
-
-	return buf;
-
-error:
-	if (NULL != proc_data)
-		rc_dynbuf_free(proc_data);
-
-	return NULL;
-}
-
+#define _SB_WRITE_STR(str) SB_WRITE(logfd, str, strlen(str), error)
 static int write_logfile(const char *logfile, const char *func, const char *path,
 						 const char *apath, const char *rpath, bool access)
 {
@@ -377,69 +322,71 @@ static int write_logfile(const char *logfile, const char *func, const char *path
 		SB_EERROR("SECURITY BREACH", "  '%s' %s\n", logfile,
 			"already exists and is not a regular file!");
 		abort();
-	} else {
-		logfd = sb_open(logfile, O_APPEND | O_WRONLY |
-				O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP |
-				S_IROTH);
-		if (logfd >= 0) {
-			char *cmdline;
-
-			if (0 != stat_ret) {
-				SB_WRITE(logfd, LOG_STRING, strlen(LOG_STRING), error);
-				SB_WRITE(logfd, LOG_FMT_FUNC, strlen(LOG_FMT_FUNC), error);
-				SB_WRITE(logfd, LOG_FMT_ACCESS, strlen(LOG_FMT_ACCESS), error);
-				SB_WRITE(logfd, LOG_FMT_PATH, strlen(LOG_FMT_PATH), error);
-				SB_WRITE(logfd, LOG_FMT_APATH, strlen(LOG_FMT_APATH), error);
-				SB_WRITE(logfd, LOG_FMT_RPATH, strlen(LOG_FMT_RPATH), error);
-				SB_WRITE(logfd, LOG_FMT_CMDLINE, strlen(LOG_FMT_CMDLINE), error);
-				SB_WRITE(logfd, "\n", 1, error);
-			} else {
-				/* Already have data in the log, so add a newline to space the
-				 * log entries.
-				 */
-				SB_WRITE(logfd, "\n", 1, error);
-			}
-
-			SB_WRITE(logfd, "F: ", 3, error);
-			SB_WRITE(logfd, func, strlen(func), error);
-			SB_WRITE(logfd, "\n", 1, error);
-			SB_WRITE(logfd, "S: ", 3, error);
-			if (access)
-				SB_WRITE(logfd, "allow", 5, error);
-			else
-				SB_WRITE(logfd, "deny", 4, error);
-			SB_WRITE(logfd, "\n", 1, error);
-			SB_WRITE(logfd, "P: ", 3, error);
-			SB_WRITE(logfd, path, strlen(path), error);
-			SB_WRITE(logfd, "\n", 1, error);
-			SB_WRITE(logfd, "A: ", 3, error);
-			SB_WRITE(logfd, apath, strlen(apath), error);
-			SB_WRITE(logfd, "\n", 1, error);
-			SB_WRITE(logfd, "R: ", 3, error);
-			SB_WRITE(logfd, rpath, strlen(rpath), error);
-			SB_WRITE(logfd, "\n", 1, error);
-
-			cmdline = getcmdline();
-			if (NULL != cmdline) {
-				SB_WRITE(logfd, "C: ", 3, error);
-				SB_WRITE(logfd, cmdline, strlen(cmdline),
-					 error);
-				SB_WRITE(logfd, "\n", 1, error);
-
-				free(cmdline);
-			} else if (0 != errno) {
-				goto error;
-			}
-
-			sb_close(logfd);
-		} else {
-			goto error;
-		}
 	}
 
+	logfd = sb_open(logfile,
+		O_APPEND | O_WRONLY | O_CREAT,
+		S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+	if (logfd == -1) {
+		SB_EERROR("ISE:write_logfile ", "unable to append logfile\n");
+		return -1;
+	}
+
+	if (0 != stat_ret)
+		_SB_WRITE_STR(
+			LOG_STRING
+			LOG_FMT_FUNC
+			LOG_FMT_ACCESS
+			LOG_FMT_PATH
+			LOG_FMT_APATH
+			LOG_FMT_RPATH
+			LOG_FMT_CMDLINE
+		);
+	/* Already have data in the log, so add a newline to space the
+	 * log entries.
+	 */
+
+	_SB_WRITE_STR("\nF: ");
+	_SB_WRITE_STR(func);
+	_SB_WRITE_STR("\nS: ");
+	if (access)
+		_SB_WRITE_STR("allow");
+	else
+		_SB_WRITE_STR("deny");
+	_SB_WRITE_STR("\nP: ");
+	_SB_WRITE_STR(path);
+	_SB_WRITE_STR("\nA: ");
+	_SB_WRITE_STR(apath);
+	_SB_WRITE_STR("\nR: ");
+	_SB_WRITE_STR(rpath);
+
+	_SB_WRITE_STR("\nC: ");
+	int cmdlinefd = sb_open(PROC_SELF_CMDLINE, O_RDONLY, 0);
+	if (cmdlinefd != -1) {
+		size_t pagesz = getpagesize();
+		char *buf = xmalloc(pagesz);
+		while (1) {
+			size_t len = sb_read(cmdlinefd, buf, pagesz);
+			if (len == -1) {
+				SB_EERROR("ISE:write_logfile ", "cmdlinefd read error\n");
+				break;
+			} else if (!len)
+				break;
+			size_t i;
+			for (i = 0; i < len; ++i)
+				if (!buf[i])
+					buf[i] = ' ';
+			SB_WRITE(logfd, buf, len, error);
+		}
+		sb_close(cmdlinefd);
+		free(buf);
+	} else
+		_SB_WRITE_STR("<unable to read " PROC_SELF_CMDLINE ">");
+	_SB_WRITE_STR("\n");
+
 	return 0;
 
-error:
+ error:
 	return -1;
 }
 
@@ -577,8 +524,13 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 		/* Fall in a read/write denied path, Deny Access */
 		goto out;
 
-	/* Hardcode denying write to log dir */
-	if (0 == strcmp(resolv_path, SANDBOX_LOG_LOCATION))
+	/* Hardcode denying write to the whole log dir.  While this is a
+	 * parial match and so rejects paths that also start with this
+	 * string, that isn't going to happen in real life so live with
+	 * it.  We can't append a slash to this path either as that would
+	 * allow people to open the dir itself for writing.
+	 */
+	if (!strncmp(resolv_path, SANDBOX_LOG_LOCATION, strlen(SANDBOX_LOG_LOCATION)))
 		goto out;
 
 	if (sbcontext->read_prefixes &&
@@ -615,7 +567,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 	    sb_nr == SB_NR_CREAT       ||
 	    sb_nr == SB_NR_CREAT64     ||
 	    sb_nr == SB_NR_MKDIR       ||
-	  /*sb_nr == SB_NR_MKNOD       ||*/
+	    sb_nr == SB_NR_MKNOD       ||
 	    sb_nr == SB_NR___XMKNOD    ||
 	    sb_nr == SB_NR_MKFIFO      ||
 	    sb_nr == SB_NR_LINK        ||
@@ -672,14 +624,13 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 			retval = check_prefixes(sbcontext->write_prefixes,
 						sbcontext->num_write_prefixes, abs_path);
 			if (1 == retval) { /* Does have write access on path */
-				char tmp_buf[SB_PATH_MAX];
-				char *dname, *rpath;
+				char *dname, *dname_buf, *rpath;
 
-				snprintf(tmp_buf, SB_PATH_MAX, "%s", abs_path);
-
-				dname = dirname(tmp_buf);
+				dname_buf = strdup(abs_path);
+				dname = dirname(dname_buf);
 				/* Get symlink resolved path */
 				rpath = resolve_path(dname, 1);
+				free(dname_buf);
 				if (NULL == rpath)
 					/* Don't really worry here about
 					 * memory issues */
@@ -700,17 +651,17 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 		}
  unlink_hack_end: ;
 
-		/* XXX: Hack to allow writing to '/proc/self/fd' (bug #91516)
-		 *      It needs to be here, as for each process '/proc/self'
-		 *      will differ ... */
+		/* Hack to allow writing to '/proc/self/fd' #91516.  It needs
+		 * to be here as for each process, the '/proc/self' symlink
+		 * will differ ...
+		 */
 		char proc_self_fd[SB_PATH_MAX];
-		if ((0 == strcmp(resolv_path, PROC_DIR)) &&
-		    (NULL != realpath(PROC_SELF_FD, proc_self_fd)))
+		if (!strncmp(resolv_path, PROC_DIR, strlen(PROC_DIR)) &&
+		    NULL != realpath(PROC_SELF_FD, proc_self_fd) &&
+		    !strncmp(resolv_path, proc_self_fd, strlen(proc_self_fd)))
 		{
-			if (0 == strcmp(resolv_path, proc_self_fd)) {
-				result = 1;
-				goto out;
-			}
+			result = 1;
+			goto out;
 		}
 
 		retval = check_prefixes(sbcontext->predict_prefixes,
@@ -722,12 +673,37 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 			goto out;
 		}
 
+		/* A very common bug (apparently) is for .py[co] files to fall out
+		 * of sync with their .py source files.  Rather than trigger a hard
+		 * failure, let's just whine about it.  Once python itself gets
+		 # sorted out, we can drop this #256953.
+		 */
+		size_t len = strlen(resolv_path);
+		if (len > 4) {
+			const char *py = resolv_path + len - 4;
+			if (!strcmp(py, ".pyc") || !strcmp(py, ".pyo")) {
+				sbcontext->show_access_violation = 0;
+				goto out;
+			}
+		}
+
 		/* If we are here, and still no joy, and its the access() call,
 		 * do not log it, but just return -1 */
 		if (sb_nr == SB_NR_ACCESS_WR) {
 			sbcontext->show_access_violation = 0;
 			goto out;
 		}
+
+		/* If operating on a location those parent dirs do not exist,
+		 * then let it through as the OS itself will trigger a fail.
+		 * This is like fopen("/foo/bar", "w") and /foo/ does not
+		 * exist.  All the functions filtered thus far fall into that
+		 * behavior category, so no need to check the syscall.
+		 */
+		char *dname_buf = strdup(resolv_path);
+		if (access(dirname(dname_buf), F_OK))
+			result = 1;
+		free(dname_buf);
 	}
 
 out:
