@@ -5,7 +5,7 @@
  * Licensed under the GPL-2
  */
 
-#define WRAPPER_ARGS_PROTO int ver, const char *pathname, __mode_t mode, __dev_t *dev
+#define WRAPPER_ARGS_PROTO int ver, const char *pathname, mode_t mode, dev_t *dev
 #define WRAPPER_ARGS ver, pathname, mode, dev
 #define WRAPPER_SAFE() FUNCTION_SANDBOX_SAFE(pathname)
 #include "__wrapper_simple.c"
