@@ -16,36 +16,32 @@
 #include "sandbox.h"
 
 static int print_debug = 0;
+#define dprintf(fmt, args...) do { if (print_debug) printf(fmt, ## args); } while (0)
+#define dputs(str) do { if (print_debug) puts(str); } while (0)
 
 volatile static int stop_called = 0;
 volatile static pid_t child_pid = 0;
 
 static char log_domain[] = "sandbox";
 static const char sandbox_banner[] = "============================= Gentoo path sandbox ==============================";
+static const char sandbox_footer[] = "--------------------------------------------------------------------------------";
 
-int setup_sandbox(struct sandbox_info_t *sandbox_info, bool interactive)
+static int setup_sandbox(struct sandbox_info_t *sandbox_info, bool interactive)
 {
 	if (NULL != getenv(ENV_PORTAGE_TMPDIR)) {
 		/* Portage handle setting SANDBOX_WRITE itself. */
 		sandbox_info->work_dir[0] = '\0';
 	} else {
 		if (NULL == getcwd(sandbox_info->work_dir, SB_PATH_MAX)) {
-			perror("sandbox:  Failed to get current directory");
+			sb_pwarn("failed to get current directory");
 			return -1;
 		}
 		if (interactive)
 			setenv(ENV_SANDBOX_WORKDIR, sandbox_info->work_dir, 1);
 	}
 
-	/* Do not resolve symlinks, etc .. libsandbox will handle that. */
-	if (!rc_is_dir(VAR_TMPDIR, true)) {
-		perror("sandbox:  Failed to get var_tmp_dir");
-		return -1;
-	}
-	snprintf(sandbox_info->var_tmp_dir, SB_PATH_MAX, "%s", VAR_TMPDIR);
-
 	if (-1 == get_tmp_dir(sandbox_info->tmp_dir)) {
-		perror("sandbox:  Failed to get tmp_dir");
+		sb_pwarn("failed to get tmp_dir");
 		return -1;
 	}
 	setenv(ENV_TMPDIR, sandbox_info->tmp_dir, 1);
@@ -66,7 +62,7 @@ int setup_sandbox(struct sandbox_info_t *sandbox_info, bool interactive)
 	get_sandbox_log(sandbox_info->sandbox_log);
 	if (rc_file_exists(sandbox_info->sandbox_log)) {
 		if (-1 == unlink(sandbox_info->sandbox_log)) {
-			perror("sandbox:  Could not unlink old log file");
+			sb_pwarn("could not unlink old log file");
 			return -1;
 		}
 	}
@@ -75,7 +71,7 @@ int setup_sandbox(struct sandbox_info_t *sandbox_info, bool interactive)
 	get_sandbox_debug_log(sandbox_info->sandbox_debug_log);
 	if (rc_file_exists(sandbox_info->sandbox_debug_log)) {
 		if (-1 == unlink(sandbox_info->sandbox_debug_log)) {
-			perror("sandbox:  Could not unlink old debug log file");
+			sb_pwarn("could not unlink old debug log file");
 			return -1;
 		}
 	}
@@ -83,49 +79,37 @@ int setup_sandbox(struct sandbox_info_t *sandbox_info, bool interactive)
 	return 0;
 }
 
-int print_sandbox_log(char *sandbox_log)
+static void print_sandbox_log(char *sandbox_log)
 {
-	int sandbox_log_file = -1;
-	char *beep_count_env = NULL;
-	int i, beep_count = 0;
-	off_t len = 0;
-	char *buffer = NULL;
-
-	if (!rc_is_file(sandbox_log, false)) {
-		perror("sandbox:  Log file is not a regular file");
-		return 0;
-	}
-
-	len = rc_get_size(sandbox_log, true);
-	if (0 == len)
-		return 0;
+	int sandbox_log_file, beep_count;
+	char *beep_count_env;
+	size_t len;
+	char buffer[1024];
 
 	sandbox_log_file = sb_open(sandbox_log, O_RDONLY, 0);
 	if (-1 == sandbox_log_file) {
-		perror("sandbox:  Could not open Log file");
-		return 0;
+		sb_pwarn("could not open og file");
+		return;
 	}
 
-	buffer = xzalloc((len + 1) * sizeof(char));
-	if (NULL == buffer) {
-		perror("sandbox:  Could not allocate buffer for Log file");
-		return 0;
-	}
-	if (-1 == sb_read(sandbox_log_file, buffer, len)) {
-		perror("sandbox:  Could read Log file");
-		return 0;
+	SB_EERROR("--------------------------- ACCESS VIOLATION SUMMARY ---------------------------", "\n");
+	SB_EERROR("LOG FILE", " \"%s\"\n\n", sandbox_log);
+
+	while (1) {
+		len = sb_read(sandbox_log_file, buffer, sizeof(buffer));
+		if (len == -1) {
+			sb_pwarn("sb_read(logfile) failed");
+			break;
+		} else if (!len)
+			break;
+		if (sb_write(STDERR_FILENO, buffer, len) != len) {
+			sb_pwarn("sb_write(logfile) failed");
+			break;
+		}
 	}
 	sb_close(sandbox_log_file);
 
-	SB_EERROR(
-	       "--------------------------- ACCESS VIOLATION SUMMARY ---------------------------",
-	       "\n");
-	SB_EERROR("LOG FILE", " \"%s\"\n\n", sandbox_log);
-	fprintf(stderr, "%s", buffer);
-	free(buffer);
-	SB_EERROR(
-	       "--------------------------------------------------------------------------------",
-	       "\n");
+	SB_EERROR("--------------------------------------------------------------------------------", "\n");
 
 	beep_count_env = getenv(ENV_SANDBOX_BEEP);
 	if (beep_count_env)
@@ -133,33 +117,27 @@ int print_sandbox_log(char *sandbox_log)
 	else
 		beep_count = DEFAULT_BEEP_COUNT;
 
-	for (i = 0; i < beep_count; i++) {
+	while (beep_count--) {
 		fputc('\a', stderr);
-		if (i < beep_count - 1)
+		if (beep_count)
 			sleep(1);
 	}
-
-	return 1;
 }
 
-void stop(int signum)
+static void stop(int signum)
 {
 	if (0 == stop_called) {
 		stop_called = 1;
-		printf("sandbox:  Caught signal %d in pid %d\n",
-		       signum, getpid());
-	} else {
-		fprintf(stderr,
-			"sandbox:  Signal already caught and busy still cleaning up!\n");
-	}
+		sb_warn("caught signal %d in pid %d\n", signum, getpid());
+	} else
+		sb_warn("signal already caught and busy still cleaning up!");
 }
 
-void usr1_handler(int signum, siginfo_t *siginfo, void *ucontext)
+static void usr1_handler(int signum, siginfo_t *siginfo, void *ucontext)
 {
 	if (0 == stop_called) {
 		stop_called = 1;
-		printf("sandbox:  Caught signal %d in pid %d\n",
-		       signum, getpid());
+		sb_warn("caught signal %d in pid %d", signum, getpid());
 
 		/* FIXME: This is really bad form, as we should kill the whole process
 		 *        tree, but currently that is too much work and not worth the
@@ -169,13 +147,11 @@ void usr1_handler(int signum, siginfo_t *siginfo, void *ucontext)
 		if (siginfo->si_pid > 0)
 			kill(siginfo->si_pid, SIGKILL);
 		kill(child_pid, SIGKILL);
-	} else {
-		fprintf(stderr,
-			"sandbox:  Signal already caught and busy still cleaning up!\n");
-	}
+	} else
+		sb_warn("signal already caught and busy still cleaning up!");
 }
 
-int spawn_shell(char *argv_bash[], char **env, int debug)
+static int spawn_shell(char *argv_bash[], char **env, int debug)
 {
 	int status = 0;
 	int ret = 0;
@@ -185,11 +161,11 @@ int spawn_shell(char *argv_bash[], char **env, int debug)
 	/* Child's process */
 	if (0 == child_pid) {
 		int ret = execve(argv_bash[0], argv_bash, env);
-		perror("sandbox:  Failed to exec child");
+		sb_pwarn("failed to exec child");
 		_exit(ret);
 	} else if (child_pid < 0) {
 		if (debug)
-			fprintf(stderr, "Process failed to spawn!\n");
+			sb_pwarn("process failed to spawn!");
 		return 0;
 	}
 
@@ -200,13 +176,13 @@ int spawn_shell(char *argv_bash[], char **env, int debug)
 
 	ret = waitpid(child_pid, &status, 0);
 	if (-1 == ret) {
-		perror("sandbox:  Failed to waitpid for child");
+		sb_pwarn("failed to waitpid for child");
 		return 0;
 	} else if (status != 0) {
 		if (WIFSIGNALED(status))
 			psignal(WTERMSIG(status), "Sandboxed process killed by signal");
 		else if (debug)
-			fprintf(stderr, "Process returned with failed exit status %d!\n", WEXITSTATUS(status));
+			sb_warn("process returned with failed exit status %d!", WEXITSTATUS(status));
 		return 0;
 	}
 
@@ -269,41 +245,31 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (print_debug)
-		puts(sandbox_banner);
+	dputs(sandbox_banner);
 
 	/* check if a sandbox is already running */
-	if (NULL != getenv(ENV_SANDBOX_ACTIVE)) {
-		fprintf(stderr, "Not launching a new sandbox instance\n");
-		fprintf(stderr, "Another one is already running in this process hierarchy.\n");
-		exit(EXIT_FAILURE);
-	}
+	if (NULL != getenv(ENV_SANDBOX_ACTIVE))
+		sb_err("not launching a new sandbox as one is already running in this process hierarchy");
 
 	/* determine the location of all the sandbox support files */
-	if (print_debug)
-		printf("Detection of the support files.\n");
+	dputs("Detection of the support files.");
 
-	if (-1 == setup_sandbox(&sandbox_info, print_debug)) {
-		fprintf(stderr, "sandbox:  Failed to setup sandbox.");
-		exit(EXIT_FAILURE);
-	}
+	if (-1 == setup_sandbox(&sandbox_info, print_debug))
+		sb_err("failed to setup sandbox");
 
 	/* verify the existance of required files */
-	if (print_debug)
-		printf("Verification of the required files.\n");
+	dputs("Verification of the required files.");
 
-	if (!rc_file_exists(sandbox_info.sandbox_rc)) {
-		perror("sandbox:  Could not open the sandbox rc file");
-		exit(EXIT_FAILURE);
-	}
+	if (!rc_file_exists(sandbox_info.sandbox_rc))
+		sb_perr("could not open the sandbox rc file");
 
 	/* set up the required environment variables */
-	if (print_debug)
-		printf("Setting up the required environment variables.\n");
+	dputs("Setting up the required environment variables.");
 
 	/* If not in portage, cd into it work directory */
 	if ('\0' != sandbox_info.work_dir[0])
-		chdir(sandbox_info.work_dir);
+		if (chdir(sandbox_info.work_dir))
+			sb_perr("chdir(%s) failed", sandbox_info.work_dir);
 
 	/* Setup the child environment stuff.
 	 * XXX:  We free this in spawn_shell(). */
@@ -327,8 +293,6 @@ int main(int argc, char **argv)
 					   (strlen(argv_bash[4]) +
 					    strlen(argv[i]) + 2) *
 					   sizeof(char));
-			if (NULL == tmp_ptr)
-				goto oom_error;
 			argv_bash[4] = tmp_ptr;
 
 			snprintf(argv_bash[4] + strlen(argv_bash[4]),
@@ -348,13 +312,9 @@ int main(int argc, char **argv)
 	sigaction (SIGUSR1, &act_new, NULL);
 
 	/* STARTING PROTECTED ENVIRONMENT */
-	if (print_debug) {
-		printf("The protected environment has been started.\n");
-		printf("--------------------------------------------------------------------------------\n");
-	}
-
-	if (print_debug)
-		printf("Process being started in forked instance.\n");
+	dputs("The protected environment has been started.");
+	dputs(sandbox_footer);
+	dputs("Process being started in forked instance.");
 
 	/* Start Bash */
 	if (!spawn_shell(argv_bash, sandbox_environ, print_debug))
@@ -366,22 +326,17 @@ int main(int argc, char **argv)
 	argv_bash = NULL;
 	sandbox_environ = NULL;
 
-	if (print_debug)
-		printf("Cleaning up sandbox process\n");
-
-	if (print_debug) {
-		puts(sandbox_banner);
-		printf("The protected environment has been shut down.\n");
-	}
+	dputs("Cleaning up sandbox process");
+	dputs(sandbox_banner);
+	dputs("The protected environment has been shut down.");
 
 	if (rc_file_exists(sandbox_info.sandbox_log)) {
 		sandbox_log_presence = 1;
 		print_sandbox_log(sandbox_info.sandbox_log);
-	} else if (print_debug) {
-		printf("--------------------------------------------------------------------------------\n");
-	}
+	} else
+		dputs(sandbox_footer);
 
-	if ((sandbox_log_presence) || (!success))
+	if (sandbox_log_presence || !success)
 		return 1;
 	else
 		return 0;
@@ -390,6 +345,5 @@ oom_error:
 	if (NULL != argv_bash)
 		str_list_free(argv_bash);
 
-	perror("sandbox:  Out of memory (environ)");
-	exit(EXIT_FAILURE);
+	sb_perr("out of memory (environ)");
 }
